/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.registry.overlay;

import java.math.BigInteger;
import java.util.Properties;
import java.util.logging.Logger;

import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.extension.exception.PetalsExtensionException;
import org.ow2.petals.registry_overlay.configuration.generated.Credentials;
import org.ow2.petals.registry_overlay.configuration.generated.Member;
import org.ow2.petals.registry_overlay.configuration.generated.Members;
import org.ow2.petals.registry_overlay.configuration.generated.PetalsRegistryOverlay;
import org.ow2.petals.registry_overlay.core.MemberConstants;
import org.ow2.petals.registry_overlay.core.RegistryOverlayNode;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * The implementation of the pre-extension 'Embedded Petals Registry Overlay'
 * 
 * @author Christophe DENEUX - Linagora
 */
@Component
public class EmbeddedRegistryOverlayServiceImpl implements EmbeddedRegistryOverlayService {

    /**
     * Credentials group name of the embedded registry
     */
    private static final String CFG_CREDENTIALS_GROUP = "default-sample";

    /**
     * Credentials group password of the embedded registry
     */
    private static final String CFG_CREDENTIALS_PWD = "s3cr3t";

    /**
     * Name of the registry member embedded
     */
    private static final String CFG_MEMBER_ID = "sample-member-0";

    /**
     * Hostname of the registry member embedded
     */
    private static final String CFG_MEMBER_HOST = "localhost";

    /**
     * The {@link ConfigurationService} of the Petals Kernel
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * The embedded Petals registry overlay node
     */
    private RegistryOverlayNode registryOverlayNode;

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(FRACTAL_COMPONENT_LOGGER_NAME));

    /**
     * The embedded Petals Registry Overlay node is created and started when starting the Fractal component
     */
    @Lifecycle(step = Step.START)
    public void start() throws PetalsExtensionException {
        this.log.start();

        try {
            this.log.info("Starting Petals Registry Overlay local member...");

            final PetalsRegistryOverlay petalsRegistryOverlayCfg = new PetalsRegistryOverlay();

            final Credentials credentials = new Credentials();
            petalsRegistryOverlayCfg.setCredentials(credentials);
            credentials.setGroup(CFG_CREDENTIALS_GROUP);
            credentials.setPassword(CFG_CREDENTIALS_PWD);

            final Members members = new Members();
            petalsRegistryOverlayCfg.setMembers(members);

            final Member member = new Member();
            members.getMember().add(member);
            member.setId(CFG_MEMBER_ID);
            member.setValue(CFG_MEMBER_HOST);
            final Properties serverProps = this.configurationService.getServerProperties();
            final String portStr = serverProps.getProperty(EmbeddedRegistryOverlayService.PORT_PROPERTY_NAME);
            if (portStr != null && !portStr.isEmpty()) {
                try {
                    final BigInteger port = BigInteger.valueOf(Long.parseLong(portStr));
                    this.log.debug("The embedded registry will be listenning on port " + port.toString());
                    member.setPort(port);
                } catch (final NumberFormatException e) {
                    this.log.warning("Invalid port value '" + portStr + "'. Default value used.");
                }
            } else {
                this.log.debug("The embedded registry will be listenning on the default port.");
            }

            final Properties memberProperties = new Properties();
            memberProperties.setProperty(MemberConstants.MEMBER_PROPERTY_NAME_MEMBER_ID, CFG_MEMBER_ID);
            memberProperties.setProperty(MemberConstants.MEMBER_PROPERTY_NAME_DISABLE_JMX_CONNECTOR,
                    Boolean.TRUE.toString());
            memberProperties.setProperty(MemberConstants.MEMBER_PROPERTY_NAME_LISTEN_HOST, CFG_MEMBER_HOST);

            this.registryOverlayNode = new RegistryOverlayNode(petalsRegistryOverlayCfg, memberProperties);
            this.registryOverlayNode.start();

            this.log.info("Petals Registry Overlay local member started.");
        } catch (final Exception e) {
            throw new PetalsExtensionException("Unexpected error", e);
        } finally {
            this.log.end();
        }
    }

    /**
     * The embedded Petals Registry Overlay node is stopped when stopping the Fractal component
     */
    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.start();

        this.log.info("Stopping Petals Registry Overlay local member...");

        this.registryOverlayNode.stop();

        this.log.info("Petals Registry Overlay local member stopped.");
        this.log.end();
    }
}
